/* The main player class
*
*Written by M.Narvidas
*/

class Player {
  PVector position,velocity; // PVector contains two floats, x and y

  Boolean isOnGround; // is the player on the ground?
  Boolean facingRight; // Track the direction, use sprite accordingly
  int animDelay; // animation timer
  int animFrame; // keeps track of which animation frame is currently shown for the player
  int coinsCollected; // a counter to keep a tally on how many coins the player has collected
  
  static final float JUMP_POWER = 11.0; // how hard the player jolts upward on jump
  static final float RUN_SPEED = 5.0; // force of player movement on ground, in pixels/cycle
  static final float AIR_RUN_SPEED = 2.0; // like run speed, but used for control while in the air
  static final float SLOWDOWN_PERC = 0.6; // friction from the ground. multiplied by the x speed each frame.
  static final float AIR_SLOWDOWN_PERC = 0.85; // resistance in the air, otherwise air control enables crazy speeds
  static final int RUN_ANIMATION_DELAY = 2; // how many game cycles pass between animation updates?
  static final float TRIVIAL_SPEED = 1.0; // if under this speed, the player is drawn as standing still
  
  Player() { // constructor
    isOnGround = false;
    facingRight = true;
    position = new PVector();
    velocity = new PVector();
    reset();
  }
  
  void reset() {
    coinsCollected = 0;
    GRAVITY_POWER = gravitation;
    healthReset = true;
    animDelay = 0;
    animFrame = 0;
    velocity.x = 0;
    velocity.y = 0;
  }
  
  void inputCheck() {
    
    if (playerReset){
      reset();
      resetGame();
      playerReset = false;
    }
    
    float speedHere = (isOnGround ? RUN_SPEED : AIR_RUN_SPEED);
    float frictionHere = (isOnGround ? SLOWDOWN_PERC : AIR_SLOWDOWN_PERC);
    
     if(leftPressed) {
      velocity.x -= speedHere;
    } else if(rightPressed) {
      velocity.x += speedHere;
    } 

    velocity.x *= frictionHere; // causes player to constantly lose speed
    
    if(isOnGround) { // player can only jump if currently on the ground
      if(upPressed) { // either up arrow or space bar cause the player to jump

        velocity.y = -JUMP_POWER; // adjust vertical speed
        isOnGround = false; // mark that the player has left the ground, i.e. cannot jump again for now
      }
    }
  }
  
  void checkForWallBumping() {
    int guyWidth = guy_stand.width; // think of image size of player standing as the player's physical size
    int guyHeight = guy_stand.height;
    int probeDistance = int(guyWidth*0.3);
    int ceilingProbeDistance = int(guyHeight*0.95);
    
    
    // Probes
    PVector leftSideHigh,rightSideHigh,leftSideLow,rightSideLow,topSide;
    leftSideHigh = new PVector();
    rightSideHigh = new PVector();
    leftSideLow = new PVector();
    rightSideLow = new PVector();
    topSide = new PVector();

    // update probes
    leftSideHigh.x = leftSideLow.x = position.x - probeDistance; // left edge of player
    rightSideHigh.x = rightSideLow.x = position.x + probeDistance; // right edge of player
    leftSideLow.y = rightSideLow.y = position.y-0.2*guyHeight; // shin high
    leftSideHigh.y = rightSideHigh.y = position.y-0.8*guyHeight; // shoulder high
    
    

    topSide.x = position.x; // center of player
    topSide.y = position.y-ceilingProbeDistance; // top of guy

    // if any edge of the player is inside a red SPIKE, reset the round
    if( theWorld.worldSquareAt(topSide)==World.TILE_SPIKE ||
         theWorld.worldSquareAt(leftSideHigh)==World.TILE_SPIKE ||
         theWorld.worldSquareAt(leftSideLow)==World.TILE_SPIKE ||
         theWorld.worldSquareAt(rightSideHigh)==World.TILE_SPIKE ||
         theWorld.worldSquareAt(rightSideLow)==World.TILE_SPIKE ||
         theWorld.worldSquareAt(position)==World.TILE_SPIKE) {
      resetGame();
      return; // any other possible collisions would be irrelevant, exit function now
    }
    
    // the following conditionals just check for collisions with each bump probe
    // depending upon which probe has collided, we push the player back the opposite direction
    
    if( theWorld.worldSquareAt(topSide)==World.TILE_SOLID || theWorld.worldSquareAt(topSide)==World.TILE_GRASS) {
      if(theWorld.worldSquareAt(position)==World.TILE_SOLID || theWorld.worldSquareAt(position)==World.TILE_GRASS) {
        position.sub(velocity);
        velocity.x=0.0;
        velocity.y=0.0;
      } else {
        position.y = theWorld.bottomOfSquare(topSide)+ceilingProbeDistance;
        if(velocity.y < 0) {
          velocity.y = 0.0;
        }
      }
    }
    
    if( theWorld.worldSquareAt(leftSideLow)==World.TILE_SOLID || theWorld.worldSquareAt(leftSideLow)==World.TILE_GRASS) {
      position.x = theWorld.rightOfSquare(leftSideLow)+probeDistance;
      if(velocity.x < 0) {
        velocity.x = 0.0;
      }
    }
   
    if( theWorld.worldSquareAt(leftSideHigh)==World.TILE_SOLID || theWorld.worldSquareAt(leftSideHigh)==World.TILE_GRASS) {
      position.x = theWorld.rightOfSquare(leftSideHigh)+probeDistance;
      if(velocity.x < 0) {
        velocity.x = 0.0;
      }
    }
   
    if( theWorld.worldSquareAt(rightSideLow)==World.TILE_SOLID || theWorld.worldSquareAt(rightSideLow)==World.TILE_GRASS) {
      position.x = theWorld.leftOfSquare(rightSideLow)-probeDistance;
      if(velocity.x > 0) {
        velocity.x = 0.0;
      }
    }
   
    if( theWorld.worldSquareAt(rightSideHigh)==World.TILE_SOLID || theWorld.worldSquareAt(rightSideHigh)==World.TILE_GRASS) {
      
      position.x = theWorld.leftOfSquare(rightSideHigh)-probeDistance;
      if(velocity.x > 0) {
        velocity.x = 0.0;
      }
    }
  }

  void checkForCoinGetting() {
    PVector centerOfPlayer;
    centerOfPlayer = new PVector(position.x,position.y-guy_stand.height/2);

    if(theWorld.worldSquareAt(centerOfPlayer)==World.TILE_COIN) {
      theWorld.setSquareAtToThis(centerOfPlayer, World.TILE_EMPTY);

      coinsCollected++;
    }
  }
  
    void checkForGravityPick() {
    PVector centerOfPlayer;
 
    centerOfPlayer = new PVector(position.x,position.y-guy_stand.height/2);

    if(theWorld.worldSquareAt(centerOfPlayer)==World.TILE_GRAVITY) {
      theWorld.setSquareAtToThis(centerOfPlayer, World.TILE_EMPTY);
  
      GRAVITY_POWER = 0.22;
    }
  }
     
   void checkEnemyCollision() {
     
    if (circleCircleIntersect(thePlayer.position.x,thePlayer.position.y, playerRadiusAOI, theEnemy.position.x, theEnemy.position.y, enemyRadiusAOI) == true) {
      textAlign(CENTER); // center align the text
      healthUpdate = -1;
  }
      
      
      
  }

  void checkForFalling() {
    // If we're standing on an empty or coin tile, we're not standing on anything. Fall!
    if(theWorld.worldSquareAt(position)==World.TILE_EMPTY ||
       theWorld.worldSquareAt(position)==World.TILE_COIN) {
       isOnGround=false;
    }
    
    if(isOnGround==false) { // not on ground?    
      if(theWorld.worldSquareAt(position)==World.TILE_SOLID || theWorld.worldSquareAt(position)==World.TILE_GRASS) { // landed on solid square?
        isOnGround = true;
        position.y = theWorld.topOfSquare(position);
        velocity.y = 0.0;
      } else { // fall
        velocity.y += GRAVITY_POWER;
      }
    }
  }

  void move() {
    position.add(velocity);
    
    checkForWallBumping();
    
    checkForCoinGetting();
    
    checkForFalling();
    
    checkForGravityPick();
    
    checkEnemyCollision();
  }
  
  void draw() {
    
    int guyWidth = guy_stand.width;
    int guyHeight = guy_stand.height;
    
    if(velocity.x<-TRIVIAL_SPEED) {
      facingRight = false;
    } else if(velocity.x>TRIVIAL_SPEED) {
      facingRight = true;
    }
    
    pushMatrix(); // transformations
    translate(position.x,position.y);
    if(facingRight==false) {
      scale(-1,1); // scale horizontally by -100%
    }
    translate(-guyWidth/2,-guyHeight); // drawing images centered on character's feet

 if(isOnGround==false) { // falling or jumping
      image(guy_run1, 0,0); // this running pose looks pretty good while in the air
    } else if(abs(velocity.x)<TRIVIAL_SPEED) { // not moving fast, i.e. standing
      image(guy_stand, 0,0);
    } else { // running. Animate.
      if(animDelay--<0) {
        animDelay=RUN_ANIMATION_DELAY;
        if(animFrame==0) {
          animFrame=1;
        } else {
          animFrame=0;
        }
      }
      
      if(animFrame==0) {
        image(guy_run1, 0,0);
      } else {
        image(guy_run2, 0,0);
      }
    }
    
    popMatrix(); // undoes all transformations
  }
}

